package com.nisum.ecomorder.client;

import com.netflix.hystrix.contrib.javanica.annotation.HystrixCommand;
import com.nisum.ecomorder.exception.ClientException;
import com.nisum.ecomorder.exception.NotFoundException;
import com.nisum.ecomorder.model.Customer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.HttpClientErrorException.NotFound;
import org.springframework.web.client.RestTemplate;

@Component
@Slf4j
public class CustomerClient {

    @Autowired
    private RestTemplate restTemplate;

    private final String baseUrl = "http://ecom-customer/";

    @HystrixCommand(fallbackMethod = "getCustomerByIdFallback")
    public Customer getCustomerById(Long id) {
        return restTemplate.getForEntity(baseUrl + "id/" + id, Customer.class).getBody();
    }

    public Customer getCustomerByIdFallback(Long id, Throwable e) {
        log.error("Error while calling ecom-product#getCustomerById with id:: {} -- cause :: {}", id, e.getMessage());
        if (e instanceof IllegalStateException) throw new ClientException();
        if (e instanceof HttpClientErrorException) {
            if (e instanceof NotFound) throw new NotFoundException("customer not found with ID: " + id);
            else throw new ClientException();
        }
        return null;
    }

}
