package com.nisum.ecomorder.service.impl;

import com.nisum.ecomorder.client.CustomerClient;
import com.nisum.ecomorder.client.ProductClient;
import com.nisum.ecomorder.exception.ClientException;
import com.nisum.ecomorder.exception.NotFoundException;
import com.nisum.ecomorder.model.Customer;
import com.nisum.ecomorder.model.Order;
import com.nisum.ecomorder.model.OrderLine;
import com.nisum.ecomorder.model.Product;
import com.nisum.ecomorder.model.dto.PlaceOrderRequestDto;
import com.nisum.ecomorder.model.mapper.OrderMapper;
import com.nisum.ecomorder.repository.OrderRepository;
import com.nisum.ecomorder.service.OrderService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class OrderServiceImpl implements OrderService {

    @Autowired
    OrderRepository orderRepository;

    @Autowired
    OrderMapper orderMapper;

    @Autowired
    ProductClient productClient;

    @Autowired
    CustomerClient customerClient;

    @Override
    public Order placeOrder(PlaceOrderRequestDto placeOrderRequestDto) {
        Order order = orderMapper.placeOrderRequestDtoToOrder(placeOrderRequestDto);
        order.setId(null);
        try {
            Customer customer = customerClient.getCustomerById(order.getCustomerId());
            order.setCustomerName(customer.getFirstName() + " " + customer.getLastName());
        } catch (ClientException e) {
            throw new ClientException("Order can't be placed due to internal errors. Please try later");
        }

        order.getLineItems().forEach(orderLine -> {
            orderLine.setId(null);
            String productId = orderLine.getProductId();
            Product product = productClient.getProductById(productId);
            if (product.getId() == null) throw new NotFoundException("product not found with ID: " + productId);
            orderLine.setProductName(product.getName());
            orderLine.setPrice(product.getPrice());
            orderLine.setSubTotal(product.getPrice() * orderLine.getQuantity());
        });
        order.setOrderTotal(order.getLineItems().stream().mapToDouble(OrderLine::getSubTotal).sum());
        return orderRepository.save(order);
    }

    @Override
    public List<Order> getAllOrders() {
        return orderRepository.findAll();
    }
}
