package com.nisum.ecomorder.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.nisum.ecomorder.model.Address;
import com.nisum.ecomorder.model.Order;
import com.nisum.ecomorder.model.dto.OrderLineDto;
import com.nisum.ecomorder.model.dto.PlaceOrderRequestDto;
import com.nisum.ecomorder.model.mapper.OrderLineMapperImpl;
import com.nisum.ecomorder.model.mapper.OrderMapper;
import com.nisum.ecomorder.model.mapper.OrderMapperImpl;
import com.nisum.ecomorder.service.OrderService;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import java.util.Arrays;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(value = {OrderController.class})
@ActiveProfiles("test")
@ContextConfiguration(classes = {OrderController.class, OrderMapperImpl.class, OrderLineMapperImpl.class})
class OrderControllerTest {

    @Autowired
    MockMvc mockMvc;

    @MockBean
    OrderService orderService;

    @Autowired
    ObjectMapper objectMapper;

    @Autowired
    OrderMapper orderMapper;

    private final String randomId = "someHexString";

    @Test
    void placeOrder() throws Exception {
        PlaceOrderRequestDto placeOrderRequestObject = getPlaceOrderRequestObject();
        Order placeOrderResponseObject = getPlaceOrderResponseObject();
        when(orderService.placeOrder(placeOrderRequestObject)).thenReturn(placeOrderResponseObject);
        String requestJson = objectMapper.writeValueAsString(placeOrderRequestObject);
        MvcResult mvcResult = mockMvc.perform(post("/place-order")
                .content(requestJson)
                .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isCreated())
                .andReturn();

        Order orderResponse = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), Order.class);
        assertNotNull(orderResponse);
        assertNotNull(orderResponse.getId());
        orderResponse.getLineItems().forEach(orderLine -> assertNotNull(orderLine.getId()));
    }

    private PlaceOrderRequestDto getPlaceOrderRequestObject() {
        PlaceOrderRequestDto placeOrderRequestDto = new PlaceOrderRequestDto();
        placeOrderRequestDto.setCustomerId(2L);
        placeOrderRequestDto.setEmail("ashokk@nisum.com");

        Address shippingAddress = new Address();
        shippingAddress.setAddressLine1("Jawaharnagar colony");
        shippingAddress.setAddressLine2("Ameerpet");
        shippingAddress.setCity("Hyderabad");
        shippingAddress.setState("TS");
        shippingAddress.setCountry("IND");
        shippingAddress.setZipCode("500001");

        placeOrderRequestDto.setShipTo(shippingAddress);

        OrderLineDto orderLineDto1 = new OrderLineDto();
        orderLineDto1.setProductId(randomId);
        orderLineDto1.setQuantity(2);

        OrderLineDto orderLineDto2 = new OrderLineDto();
        orderLineDto2.setProductId(randomId);
        orderLineDto2.setQuantity(2);

        placeOrderRequestDto.setLineItems(Arrays.asList(orderLineDto1, orderLineDto2));

        return placeOrderRequestDto;
    }

    private Order getPlaceOrderResponseObject() {
        Order order = orderMapper.placeOrderRequestDtoToOrder(getPlaceOrderRequestObject());
        order.setId(randomId);
//       order.setOrderDate(LocalDateTime.now());
        order.setCustomerName("Ashok kumar");
        order.getLineItems().forEach(orderLine -> {
            orderLine.setId(randomId);
            orderLine.setProductName("new bean bag");
            orderLine.setPrice(1250d);
            orderLine.setSubTotal(2500d);
        });

        order.setOrderTotal(5000D);

        return order;
    }

}